# -*- coding: UTF-8 -*-
# (c)2013 Mik Kocikowski, MIT License (http://opensource.org/licenses/MIT)
# https://github.com/mkocikowski/esbench

import datetime
import os.path
import unittest
import json
import logging

import esbench.analyze
import esbench.api
import esbench.test.test_api


class ApiCallsTest(unittest.TestCase):

    def test_get_benchmarks(self):
        conn = esbench.api.Conn(conn_cls=esbench.test.test_api.MockHTTPConnection)
        resp = esbench.analyze._get_benchmarks(conn)
        self.assertEqual(resp.curl, 'curl -XGET http://localhost:9200/%s/bench/_search?sort=benchmark_start:asc&size=100' % esbench.STATS_INDEX_NAME)


    def test_get_observations(self):
        conn = esbench.api.Conn(conn_cls=esbench.test.test_api.MockHTTPConnection)
        resp = esbench.analyze._get_observations(conn=conn, benchmark_id='foo')
        self.assertEqual(resp.curl, 'curl -XGET http://localhost:9200/%s/obs/_search?q=meta.benchmark_id:foo&sort=meta.observation_start:asc&size=10000' % (esbench.STATS_INDEX_NAME, ))


class DataTest(unittest.TestCase):

    def setUp(self):
        self.tmp_get_benchmarks = esbench.analyze._get_benchmarks
        esbench.analyze._get_benchmarks = lambda **kwargs: esbench.api.ApiResponse(
            status=200,
            reason='OK',
            data = r"""{"took":0,"timed_out":false,"_shards":{"total":1,"successful":1,"failed":0},"hits":{"total":1,"max_score":null,"hits":[{"_index":"esbench_stats","_type":"bench","_id":"3e878942","_score":null, "_source" : {"cluster": {"cluster_name": "elasticsearch", "nodes": {"TqybOUCJRPOvLdshFYQqtg": {"hostname": "MK.local", "http": {"bound_address": "inet[/0:0:0:0:0:0:0:0%0:9200]", "max_content_length": "100mb", "max_content_length_in_bytes": 104857600, "publish_address": "inet[/192.168.1.100:9200]"}, "http_address": "inet[/192.168.1.100:9200]", "jvm": {"mem": {"direct_max": "1019.8mb", "direct_max_in_bytes": 1069416448, "heap_init": "256mb", "heap_init_in_bytes": 268435456, "heap_max": "1019.8mb", "heap_max_in_bytes": 1069416448, "non_heap_init": "23.1mb", "non_heap_init_in_bytes": 24317952, "non_heap_max": "130mb", "non_heap_max_in_bytes": 136314880}, "pid": 15348, "start_time": 1388943758533, "version": "1.6.0_65", "vm_name": "Java HotSpot(TM) 64-Bit Server VM", "vm_vendor": "Apple Inc.", "vm_version": "20.65-b04-462"}, "name": "Space Turnip", "network": {"primary_interface": {"address": "192.168.1.100", "mac_address": "F8:1E:DF:D7:B4:4A", "name": "en1"}, "refresh_interval": 5000}, "os": {"available_processors": 2, "cpu": {"cache_size": "3kb", "cache_size_in_bytes": 3072, "cores_per_socket": 2, "mhz": 2260, "model": "MacBookPro5,5", "total_cores": 2, "total_sockets": 1, "vendor": "Intel"}, "mem": {"total": "4gb", "total_in_bytes": 4294967296}, "refresh_interval": 1000, "swap": {"total": "1gb", "total_in_bytes": 1073741824}}, "plugins": [{"description": "No description found for head.", "jvm": false, "name": "head", "site": true, "url": "/_plugin/head/"}], "process": {"id": 15348, "max_file_descriptors": 10240, "refresh_interval": 1000}, "settings": {"cluster.name": "elasticsearch", "foreground": "yes", "name": "Space Turnip", "path.home": "/Users/mkocikowski/dev/elasticsearch-0.90.6", "path.logs": "/Users/mkocikowski/dev/elasticsearch-0.90.6/logs"}, "thread_pool": {"bulk": {"max": 2, "min": 2, "queue_size": "50", "type": "fixed"}, "flush": {"keep_alive": "5m", "max": 1, "min": 1, "type": "scaling"}, "generic": {"keep_alive": "30s", "type": "cached"}, "get": {"max": 2, "min": 2, "queue_size": "1k", "type": "fixed"}, "index": {"max": 2, "min": 2, "queue_size": "200", "type": "fixed"}, "management": {"keep_alive": "5m", "max": 5, "min": 1, "type": "scaling"}, "merge": {"keep_alive": "5m", "max": 1, "min": 1, "type": "scaling"}, "optimize": {"max": 1, "min": 1, "type": "fixed"}, "percolate": {"max": 2, "min": 2, "queue_size": "1k", "type": "fixed"}, "refresh": {"keep_alive": "5m", "max": 1, "min": 1, "type": "scaling"}, "search": {"max": 6, "min": 6, "queue_size": "1k", "type": "fixed"}, "snapshot": {"keep_alive": "5m", "max": 1, "min": 1, "type": "scaling"}, "suggest": {"max": 2, "min": 2, "queue_size": "1k", "type": "fixed"}, "warmer": {"keep_alive": "5m", "max": 1, "min": 1, "type": "scaling"}}, "transport": {"bound_address": "inet[/0:0:0:0:0:0:0:0%0:9300]", "publish_address": "inet[/192.168.1.100:9300]"}, "transport_address": "inet[/192.168.1.100:9300]", "version": "0.90.6"}}, "ok": true}, "meta": {"argv": {"append": false, "command": "run", "config_file_path": "/Users/mkocikowski/dev/esbench/esbench/config.json", "data": null, "maxsize": "1mb", "name": "MK.local::2014-01-05T18:06:06Z", "no_load": false, "no_optimize_calls": false, "observations": 10, "record_segments": false, "repetitions": 100, "segments": null, "verbose": false}, "benchmark_id": "3e878942", "benchmark_name": "MK.local::2014-01-05T18:06:06Z", "benchmark_start": "2014-01-05T18:06:06Z", "benchmark_stop": "2014-01-05T18:06:30Z", "config": "{\"config\": {\"name_doctype\": \"doc\", \"name_index\": \"esbench_test\"}, \"index\": {\"mappings\": {\"doc\": {\"_size\": {\"enabled\": true, \"store\": \"yes\"}, \"_source\": {\"enabled\": true}, \"properties\": {\"abstract\": {\"store\": \"yes\", \"type\": \"string\"}}}}, \"settings\": {\"index\": {\"number_of_replicas\": 0, \"number_of_shards\": 1}}}, \"queries\": {\"match_description\": {\"fields\": [\"description\"], \"from\": 0, \"query\": {\"match\": {\"description\": \"computing V%(variable)s device portable\"}}, \"size\": 10}, \"match_description_facet_date_histogram\": {\"facets\": {\"date_published\": {\"date_histogram\": {\"field\": \"dates.date_published\", \"interval\": \"month\"}}}, \"fields\": [\"description\"], \"from\": 0, \"query\": {\"match\": {\"description\": \"computing V%(variable)s device portable\"}}, \"size\": 10}, \"match_description_sorted_abstract\": {\"fields\": [\"description\"], \"from\": 0, \"query\": {\"match\": {\"description\": \"computing V%(variable)s device portable\"}}, \"size\": 10, \"sort\": [{\"abstract\": \"desc\"}]}}}", "t_total": "0.41m", "t_total_in_millis": 24757}},"sort":[1388945166000]}]}}""",
            curl='curl "http://localhost:9200/esbench_stats/bench/_search?sort=benchmark_start:asc&size=100"',
        )
        self.tmp_get_observations = esbench.analyze._get_observations
        esbench.analyze._get_observations = lambda **kwargs: esbench.api.ApiResponse(
            status=200,
            reason='OK',
#             data='{"took":2,"timed_out":false,"_shards":{"total":1,"successful":1,"failed":0},"hits":{"total":10,"max_score":null,"hits":[{"_index":"esbench_stats","_type":"obs","_id":"6ae5af7f","_score":null, "_source" : {"cluster": {"cluster_name": "elasticsearch", "nodes": {"0rNB3lMgQ3elZO7IZMROZg": {"fs": {"data": [{"available": "172.1gb", "available_in_bytes": 184896188416, "dev": "/dev/disk0s2", "disk_io_op": 8134147, "disk_io_size": "1913.4gb", "disk_io_size_in_bytes": 2054519834624, "disk_read_size": "796.4gb", "disk_read_size_in_bytes": 855137120256, "disk_reads": 3031481, "disk_write_size": "1117gb", "disk_write_size_in_bytes": 1199382714368, "disk_writes": 5102666, "free": "172.4gb", "free_in_bytes": 185158332416, "mount": "/", "path": "/Users/z013rqg/dev/elasticsearch-0.90.8/data/elasticsearch/nodes/0", "total": "233.4gb", "total_in_bytes": 250656219136}], "timestamp": 1388615489634, "total": {"available": "172.1gb", "available_in_bytes": 184896188416, "disk_io_op": 8134147, "disk_io_size": "1913.4gb", "disk_io_size_in_bytes": 2054519834624, "disk_read_size": "796.4gb", "disk_read_size_in_bytes": 855137120256, "disk_reads": 3031481, "disk_write_size": "1117gb", "disk_write_size_in_bytes": 1199382714368, "disk_writes": 5102666, "free": "172.4gb", "free_in_bytes": 185158332416, "total": "233.4gb", "total_in_bytes": 250656219136}}, "hostname": "14109fd2699b.local", "http": {"current_open": 1, "total_opened": 98}, "indices": {"completion": {"size": "0b", "size_in_bytes": 0}, "docs": {"count": 239829, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "52.6kb", "memory_size_in_bytes": 53952}, "dates.date_published": {"memory_size": "96b", "memory_size_in_bytes": 96}}, "memory_size": "52.7kb", "memory_size_in_bytes": 54048}, "filter_cache": {"evictions": 0, "memory_size": "48b", "memory_size_in_bytes": 48}, "flush": {"total": 416, "total_time": "1.9s", "total_time_in_millis": 1993}, "get": {"current": 0, "exists_time": "2ms", "exists_time_in_millis": 2, "exists_total": 8, "get_time": "2ms", "missing_time": "0s", "missing_time_in_millis": 0, "missing_total": 1, "time_in_millis": 2, "total": 9}, "id_cache": {"memory_size": "0b", "memory_size_in_bytes": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "2.3m", "index_time_in_millis": 139420, "index_total": 281312}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 37, "total_docs": 103136, "total_size": "138.9mb", "total_size_in_bytes": 145682835, "total_time": "18.3s", "total_time_in_millis": 18349}, "refresh": {"total": 3618, "total_time": "1.4m", "total_time_in_millis": 84817}, "search": {"fetch_current": 0, "fetch_time": "47.6s", "fetch_time_in_millis": 47699, "fetch_total": 39737, "open_contexts": 0, "query_current": 0, "query_time": "14.2s", "query_time_in_millis": 14219, "query_total": 39737}, "segments": {"count": 26}, "store": {"size": "287.3mb", "size_in_bytes": 301270141, "throttle_time": "5.2m", "throttle_time_in_millis": 317287}, "warmer": {"current": 0, "total": 3758, "total_time": "1.1s", "total_time_in_millis": 1103}}, "jvm": {"gc": {"collection_count": 23540, "collection_time": "47.3s", "collection_time_in_millis": 47306, "collectors": {"ConcurrentMarkSweep": {"collection_count": 50, "collection_time": "236ms", "collection_time_in_millis": 236}, "ParNew": {"collection_count": 23490, "collection_time": "47s", "collection_time_in_millis": 47070}}}, "mem": {"heap_committed": "254.6mb", "heap_committed_in_bytes": 267059200, "heap_max": "1011.2mb", "heap_max_in_bytes": 1060372480, "heap_used": "159.1mb", "heap_used_in_bytes": 166835000, "heap_used_percent": 15, "non_heap_committed": "82.4mb", "non_heap_committed_in_bytes": 86437888, "non_heap_used": "53mb", "non_heap_used_in_bytes": 55606888, "pools": {"CMS Old Gen": {"max": "896mb", "max_in_bytes": 939524096, "peak_max": "896mb", "peak_max_in_bytes": 939524096, "peak_used": "195.7mb", "peak_used_in_bytes": 205260360, "used": "152.5mb", "used_in_bytes": 160008408}, "CMS Perm Gen": {"max": "82mb", "max_in_bytes": 85983232, "peak_max": "82mb", "peak_max_in_bytes": 85983232, "peak_used": "43.8mb", "peak_used_in_bytes": 45966120, "used": "43.8mb", "used_in_bytes": 45966120}, "Code Cache": {"max": "48mb", "max_in_bytes": 50331648, "peak_max": "48mb", "peak_max_in_bytes": 50331648, "peak_used": "9.2mb", "peak_used_in_bytes": 9676288, "used": "9.1mb", "used_in_bytes": 9640768}, "Par Eden Space": {"max": "102.5mb", "max_in_bytes": 107479040, "peak_max": "102.5mb", "peak_max_in_bytes": 107479040, "peak_used": "16.6mb", "peak_used_in_bytes": 17432576, "used": "6.4mb", "used_in_bytes": 6721952}, "Par Survivor Space": {"max": "12.7mb", "max_in_bytes": 13369344, "peak_max": "12.7mb", "peak_max_in_bytes": 13369344, "peak_used": "2mb", "peak_used_in_bytes": 2162688, "used": "102.1kb", "used_in_bytes": 104640}}}, "threads": {"count": 113, "peak_count": 128}, "timestamp": 1388615489634, "uptime": "1.1d", "uptime_in_millis": 101172863}, "name": "Mace, Gideon", "network": {"tcp": {"active_opens": 95394, "attempt_fails": 442, "curr_estab": 92, "estab_resets": 1227, "in_errs": 282, "in_segs": 74566315, "out_rsts": -1, "out_segs": 73220868, "passive_opens": 1136, "retrans_segs": 12816}}, "os": {"cpu": {"idle": 94, "stolen": 0, "sys": 2, "usage": 4, "user": 2}, "load_average": [0.5927734375, 0.68505859375, 0.66064453125], "mem": {"actual_free": "8gb", "actual_free_in_bytes": 8644182016, "actual_used": "7.9gb", "actual_used_in_bytes": 8535687168, "free": "6.6gb", "free_in_bytes": 7093882880, "free_percent": 50, "used": "9.3gb", "used_in_bytes": 10085986304, "used_percent": 49}, "swap": {"free": "1gb", "free_in_bytes": 1157390336, "used": "944.2mb", "used_in_bytes": 990093312}, "timestamp": 1388615489633, "uptime": "17.3m", "uptime_in_millis": 1039002}, "process": {"cpu": {"percent": 3, "sys": "6.7m", "sys_in_millis": 406567, "total": "25.4m", "total_in_millis": 1527520, "user": "18.6m", "user_in_millis": 1120953}, "mem": {"resident": "459.1mb", "resident_in_bytes": 481472512, "share": "-1b", "share_in_bytes": -1, "total_virtual": "3.7gb", "total_virtual_in_bytes": 4048965632}, "open_file_descriptors": 343, "timestamp": 1388615489633}, "thread_pool": {"bulk": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "flush": {"active": 0, "completed": 416, "largest": 3, "queue": 0, "rejected": 0, "threads": 0}, "generic": {"active": 0, "completed": 10497, "largest": 4, "queue": 0, "rejected": 0, "threads": 3}, "get": {"active": 0, "completed": 9, "largest": 8, "queue": 0, "rejected": 0, "threads": 8}, "index": {"active": 0, "completed": 341852, "largest": 8, "queue": 0, "rejected": 0, "threads": 8}, "management": {"active": 1, "completed": 7536, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "merge": {"active": 0, "completed": 3659, "largest": 4, "queue": 0, "rejected": 0, "threads": 1}, "optimize": {"active": 0, "completed": 193, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "percolate": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "refresh": {"active": 0, "completed": 3582, "largest": 4, "queue": 0, "rejected": 0, "threads": 1}, "search": {"active": 0, "completed": 40422, "largest": 24, "queue": 0, "rejected": 0, "threads": 24}, "snapshot": {"active": 0, "completed": 768, "largest": 4, "queue": 0, "rejected": 0, "threads": 1}, "suggest": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "warmer": {"active": 0, "completed": 4250, "largest": 2, "queue": 0, "rejected": 0, "threads": 1}}, "timestamp": 1388615490536, "transport": {"rx_count": 0, "rx_size": "0b", "rx_size_in_bytes": 0, "server_open": 13, "tx_count": 0, "tx_size": "0b", "tx_size_in_bytes": 0}, "transport_address": "inet[/192.168.1.100:9300]"}}}, "meta": {"benchmark_id": "9eed3105", "observation_id": "6ae5af7f", "observation_sequence_no": 2, "observation_start": "2014-01-01T22:31:29Z", "observation_stop": "2014-01-01T22:31:30Z"}, "segments": {"num_committed_segments": 2, "num_search_segments": 2, "segments": null, "t_optimize": "0.02s", "t_optimize_in_millis": 18}, "stats": {"docs": {"count": 11, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "52.6kb", "memory_size_in_bytes": 53952}, "dates.date_published": {"memory_size": "96b", "memory_size_in_bytes": 96}}, "memory_size": "52.7kb", "memory_size_in_bytes": 54048}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "32ms", "index_time_in_millis": 32, "index_total": 11}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "446ms", "fetch_time_in_millis": 446, "fetch_total": 600, "groups": {"match_description": {"client_time": "0.27s", "client_time_in_millis": 269, "client_total": 100, "fetch_current": 0, "fetch_time": "100ms", "fetch_time_in_millis": 100, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}, "match_description_facet_date_histogram": {"client_time": "0.28s", "client_time_in_millis": 277, "client_total": 100, "fetch_current": 0, "fetch_time": "103ms", "fetch_time_in_millis": 103, "fetch_total": 100, "query_current": 0, "query_time": "21ms", "query_time_in_millis": 21, "query_total": 100}, "match_description_sorted_abstract": {"client_time": "0.30s", "client_time_in_millis": 297, "client_total": 100, "fetch_current": 0, "fetch_time": "107ms", "fetch_time_in_millis": 107, "fetch_total": 100, "query_current": 0, "query_time": "26ms", "query_time_in_millis": 26, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "114ms", "query_time_in_millis": 114, "query_total": 600}, "store": {"size": "294.1kb", "size_in_bytes": 301249, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1388615489000]},{"_index":"esbench_stats","_type":"obs","_id":"4391531c","_score":null, "_source" : {"cluster": {"cluster_name": "elasticsearch", "nodes": {"0rNB3lMgQ3elZO7IZMROZg": {"fs": {"data": [{"available": "172.1gb", "available_in_bytes": 184896188416, "dev": "/dev/disk0s2", "disk_io_op": 8134147, "disk_io_size": "1913.4gb", "disk_io_size_in_bytes": 2054519834624, "disk_read_size": "796.4gb", "disk_read_size_in_bytes": 855137120256, "disk_reads": 3031481, "disk_write_size": "1117gb", "disk_write_size_in_bytes": 1199382714368, "disk_writes": 5102666, "free": "172.4gb", "free_in_bytes": 185158332416, "mount": "/", "path": "/Users/z013rqg/dev/elasticsearch-0.90.8/data/elasticsearch/nodes/0", "total": "233.4gb", "total_in_bytes": 250656219136}], "timestamp": 1388615489634, "total": {"available": "172.1gb", "available_in_bytes": 184896188416, "disk_io_op": 8134147, "disk_io_size": "1913.4gb", "disk_io_size_in_bytes": 2054519834624, "disk_read_size": "796.4gb", "disk_read_size_in_bytes": 855137120256, "disk_reads": 3031481, "disk_write_size": "1117gb", "disk_write_size_in_bytes": 1199382714368, "disk_writes": 5102666, "free": "172.4gb", "free_in_bytes": 185158332416, "total": "233.4gb", "total_in_bytes": 250656219136}}, "hostname": "14109fd2699b.local", "http": {"current_open": 1, "total_opened": 98}, "indices": {"completion": {"size": "0b", "size_in_bytes": 0}, "docs": {"count": 239823, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "26.4kb", "memory_size_in_bytes": 27086}, "dates.date_published": {"memory_size": "48b", "memory_size_in_bytes": 48}}, "memory_size": "26.4kb", "memory_size_in_bytes": 27134}, "filter_cache": {"evictions": 0, "memory_size": "24b", "memory_size_in_bytes": 24}, "flush": {"total": 416, "total_time": "1.9s", "total_time_in_millis": 1993}, "get": {"current": 0, "exists_time": "2ms", "exists_time_in_millis": 2, "exists_total": 8, "get_time": "2ms", "missing_time": "0s", "missing_time_in_millis": 0, "missing_total": 1, "time_in_millis": 2, "total": 9}, "id_cache": {"memory_size": "0b", "memory_size_in_bytes": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "2.3m", "index_time_in_millis": 139399, "index_total": 281306}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 37, "total_docs": 103136, "total_size": "138.9mb", "total_size_in_bytes": 145682835, "total_time": "18.3s", "total_time_in_millis": 18349}, "refresh": {"total": 3617, "total_time": "1.4m", "total_time_in_millis": 84785}, "search": {"fetch_current": 0, "fetch_time": "47.3s", "fetch_time_in_millis": 47386, "fetch_total": 39437, "open_contexts": 0, "query_current": 0, "query_time": "14.1s", "query_time_in_millis": 14148, "query_total": 39437}, "segments": {"count": 24}, "store": {"size": "287mb", "size_in_bytes": 300994510, "throttle_time": "5.2m", "throttle_time_in_millis": 317287}, "warmer": {"current": 0, "total": 3756, "total_time": "1.1s", "total_time_in_millis": 1103}}, "jvm": {"gc": {"collection_count": 23540, "collection_time": "47.3s", "collection_time_in_millis": 47306, "collectors": {"ConcurrentMarkSweep": {"collection_count": 50, "collection_time": "236ms", "collection_time_in_millis": 236}, "ParNew": {"collection_count": 23490, "collection_time": "47s", "collection_time_in_millis": 47070}}}, "mem": {"heap_committed": "254.6mb", "heap_committed_in_bytes": 267059200, "heap_max": "1011.2mb", "heap_max_in_bytes": 1060372480, "heap_used": "159.1mb", "heap_used_in_bytes": 166835000, "heap_used_percent": 15, "non_heap_committed": "82.4mb", "non_heap_committed_in_bytes": 86437888, "non_heap_used": "53mb", "non_heap_used_in_bytes": 55606888, "pools": {"CMS Old Gen": {"max": "896mb", "max_in_bytes": 939524096, "peak_max": "896mb", "peak_max_in_bytes": 939524096, "peak_used": "195.7mb", "peak_used_in_bytes": 205260360, "used": "152.5mb", "used_in_bytes": 160008408}, "CMS Perm Gen": {"max": "82mb", "max_in_bytes": 85983232, "peak_max": "82mb", "peak_max_in_bytes": 85983232, "peak_used": "43.8mb", "peak_used_in_bytes": 45966120, "used": "43.8mb", "used_in_bytes": 45966120}, "Code Cache": {"max": "48mb", "max_in_bytes": 50331648, "peak_max": "48mb", "peak_max_in_bytes": 50331648, "peak_used": "9.2mb", "peak_used_in_bytes": 9676288, "used": "9.1mb", "used_in_bytes": 9640768}, "Par Eden Space": {"max": "102.5mb", "max_in_bytes": 107479040, "peak_max": "102.5mb", "peak_max_in_bytes": 107479040, "peak_used": "16.6mb", "peak_used_in_bytes": 17432576, "used": "6.4mb", "used_in_bytes": 6721952}, "Par Survivor Space": {"max": "12.7mb", "max_in_bytes": 13369344, "peak_max": "12.7mb", "peak_max_in_bytes": 13369344, "peak_used": "2mb", "peak_used_in_bytes": 2162688, "used": "102.1kb", "used_in_bytes": 104640}}}, "threads": {"count": 113, "peak_count": 128}, "timestamp": 1388615489634, "uptime": "1.1d", "uptime_in_millis": 101172863}, "name": "Mace, Gideon", "network": {"tcp": {"active_opens": 95394, "attempt_fails": 442, "curr_estab": 92, "estab_resets": 1227, "in_errs": 282, "in_segs": 74566315, "out_rsts": -1, "out_segs": 73220868, "passive_opens": 1136, "retrans_segs": 12816}}, "os": {"cpu": {"idle": 94, "stolen": 0, "sys": 2, "usage": 4, "user": 2}, "load_average": [0.5927734375, 0.68505859375, 0.66064453125], "mem": {"actual_free": "8gb", "actual_free_in_bytes": 8644182016, "actual_used": "7.9gb", "actual_used_in_bytes": 8535687168, "free": "6.6gb", "free_in_bytes": 7093882880, "free_percent": 50, "used": "9.3gb", "used_in_bytes": 10085986304, "used_percent": 49}, "swap": {"free": "1gb", "free_in_bytes": 1157390336, "used": "944.2mb", "used_in_bytes": 990093312}, "timestamp": 1388615489633, "uptime": "17.3m", "uptime_in_millis": 1039002}, "process": {"cpu": {"percent": 3, "sys": "6.7m", "sys_in_millis": 406567, "total": "25.4m", "total_in_millis": 1527520, "user": "18.6m", "user_in_millis": 1120953}, "mem": {"resident": "459.1mb", "resident_in_bytes": 481472512, "share": "-1b", "share_in_bytes": -1, "total_virtual": "3.7gb", "total_virtual_in_bytes": 4048965632}, "open_file_descriptors": 343, "timestamp": 1388615489633}, "thread_pool": {"bulk": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "flush": {"active": 0, "completed": 416, "largest": 3, "queue": 0, "rejected": 0, "threads": 0}, "generic": {"active": 0, "completed": 10495, "largest": 4, "queue": 0, "rejected": 0, "threads": 3}, "get": {"active": 0, "completed": 9, "largest": 8, "queue": 0, "rejected": 0, "threads": 8}, "index": {"active": 0, "completed": 341846, "largest": 8, "queue": 0, "rejected": 0, "threads": 8}, "management": {"active": 1, "completed": 7532, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "merge": {"active": 0, "completed": 3658, "largest": 4, "queue": 0, "rejected": 0, "threads": 1}, "optimize": {"active": 0, "completed": 192, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "percolate": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "refresh": {"active": 0, "completed": 3581, "largest": 4, "queue": 0, "rejected": 0, "threads": 1}, "search": {"active": 0, "completed": 40122, "largest": 24, "queue": 0, "rejected": 0, "threads": 24}, "snapshot": {"active": 0, "completed": 768, "largest": 4, "queue": 0, "rejected": 0, "threads": 1}, "suggest": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "warmer": {"active": 0, "completed": 4248, "largest": 2, "queue": 0, "rejected": 0, "threads": 1}}, "timestamp": 1388615489633, "transport": {"rx_count": 0, "rx_size": "0b", "rx_size_in_bytes": 0, "server_open": 13, "tx_count": 0, "tx_size": "0b", "tx_size_in_bytes": 0}, "transport_address": "inet[/192.168.1.100:9300]"}}}, "meta": {"benchmark_id": "9eed3105", "observation_id": "4391531c", "observation_sequence_no": 1, "observation_start": "2014-01-01T22:31:29Z", "observation_stop": "2014-01-01T22:31:29Z"}, "segments": {"num_committed_segments": 1, "num_search_segments": 1, "segments": null, "t_optimize": "0.02s", "t_optimize_in_millis": 18}, "stats": {"docs": {"count": 6, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "26.4kb", "memory_size_in_bytes": 27086}, "dates.date_published": {"memory_size": "48b", "memory_size_in_bytes": 48}}, "memory_size": "26.4kb", "memory_size_in_bytes": 27134}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "18ms", "index_time_in_millis": 18, "index_total": 6}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "133ms", "fetch_time_in_millis": 133, "fetch_total": 300, "groups": {"match_description": {"client_time": "0.17s", "client_time_in_millis": 172, "client_total": 100, "fetch_current": 0, "fetch_time": "42ms", "fetch_time_in_millis": 42, "fetch_total": 100, "query_current": 0, "query_time": "15ms", "query_time_in_millis": 15, "query_total": 100}, "match_description_facet_date_histogram": {"client_time": "0.18s", "client_time_in_millis": 176, "client_total": 100, "fetch_current": 0, "fetch_time": "48ms", "fetch_time_in_millis": 48, "fetch_total": 100, "query_current": 0, "query_time": "12ms", "query_time_in_millis": 12, "query_total": 100}, "match_description_sorted_abstract": {"client_time": "0.18s", "client_time_in_millis": 178, "client_total": 100, "fetch_current": 0, "fetch_time": "43ms", "fetch_time_in_millis": 43, "fetch_total": 100, "query_current": 0, "query_time": "15ms", "query_time_in_millis": 15, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "43ms", "query_time_in_millis": 43, "query_total": 300}, "store": {"size": "149.7kb", "size_in_bytes": 153296, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1388615489000]}]}}',
#             data='{"took":2,"timed_out":false,"_shards":{"total":1,"successful":1,"failed":0},"hits":{"total":10,"max_score":null,"hits":[{"_index":"esbench_stats","_type":"obs","_id":"e8cd3f18","_score":null, "_source" : {"cluster": {"cluster_name": "elasticsearch", "nodes": {"TqybOUCJRPOvLdshFYQqtg": {"fs": {"data": [{"available": "58gb", "available_in_bytes": 62311968768, "dev": "/dev/disk1s2", "disk_read_size": "4.3gb", "disk_read_size_in_bytes": 4703711744, "disk_reads": 248828, "disk_write_size": "15gb", "disk_write_size_in_bytes": 16135558656, "disk_writes": 468533, "free": "58gb", "free_in_bytes": 62311968768, "mount": "/Users/mkocikowski", "path": "/Users/mkocikowski/dev/elasticsearch-0.90.6/data/elasticsearch/nodes/0", "total": "465.1gb", "total_in_bytes": 499430719488}], "timestamp": 1388945167521}, "hostname": "MK.local", "http": {"current_open": 2, "total_opened": 10}, "indices": {"completion": {"size": "0b", "size_in_bytes": 0}, "docs": {"count": 6, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "26.4kb", "memory_size_in_bytes": 27086}, "dates.date_published": {"memory_size": "48b", "memory_size_in_bytes": 48}}, "memory_size": "26.4kb", "memory_size_in_bytes": 27134}, "filter_cache": {"evictions": 0, "memory_size": "24b", "memory_size_in_bytes": 24}, "flush": {"total": 0, "total_time": "0s", "total_time_in_millis": 0}, "get": {"current": 0, "exists_time": "1ms", "exists_time_in_millis": 1, "exists_total": 6, "get_time": "1ms", "missing_time": "0s", "missing_time_in_millis": 0, "missing_total": 0, "time_in_millis": 1, "total": 6}, "id_cache": {"memory_size": "0b", "memory_size_in_bytes": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "2.5s", "index_time_in_millis": 2558, "index_total": 147}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 6, "total_docs": 149, "total_size": "8.6mb", "total_size_in_bytes": 9066343, "total_time": "3s", "total_time_in_millis": 3053}, "refresh": {"total": 43, "total_time": "7.8s", "total_time_in_millis": 7826}, "search": {"fetch_current": 0, "fetch_time": "28.5s", "fetch_time_in_millis": 28550, "fetch_total": 9300, "open_contexts": 0, "query_current": 0, "query_time": "8.6s", "query_time_in_millis": 8629, "query_total": 9300}, "store": {"size": "149.6kb", "size_in_bytes": 153268, "throttle_time": "0s", "throttle_time_in_millis": 0}, "warmer": {"current": 0, "total": 3, "total_time": "0s", "total_time_in_millis": 0}}, "jvm": {"gc": {"collection_count": 1179, "collection_time": "3.9s", "collection_time_in_millis": 3911, "collectors": {"ConcurrentMarkSweep": {"collection_count": 0, "collection_time": "0s", "collection_time_in_millis": 0}, "ParNew": {"collection_count": 1179, "collection_time": "3.9s", "collection_time_in_millis": 3911}}}, "mem": {"heap_committed": "253.9mb", "heap_committed_in_bytes": 266272768, "heap_used": "137.8mb", "heap_used_in_bytes": 144510592, "non_heap_committed": "46.2mb", "non_heap_committed_in_bytes": 48500736, "non_heap_used": "46mb", "non_heap_used_in_bytes": 48261344, "pools": {"CMS Old Gen": {"max": "982.4mb", "max_in_bytes": 1030160384, "peak_max": "982.4mb", "peak_max_in_bytes": 1030160384, "peak_used": "128.7mb", "peak_used_in_bytes": 135035472, "used": "128.7mb", "used_in_bytes": 135035472}, "CMS Perm Gen": {"max": "82mb", "max_in_bytes": 85983232, "peak_max": "82mb", "peak_max_in_bytes": 85983232, "peak_used": "41.2mb", "peak_used_in_bytes": 43263328, "used": "41.2mb", "used_in_bytes": 43263328}, "Code Cache": {"max": "48mb", "max_in_bytes": 50331648, "peak_max": "48mb", "peak_max_in_bytes": 50331648, "peak_used": "4.7mb", "peak_used_in_bytes": 5008832, "used": "4.7mb", "used_in_bytes": 4998016}, "Par Eden Space": {"max": "33.3mb", "max_in_bytes": 34930688, "peak_max": "33.3mb", "peak_max_in_bytes": 34930688, "peak_used": "16.6mb", "peak_used_in_bytes": 17432576, "used": "9mb", "used_in_bytes": 9455216}, "Par Survivor Space": {"max": "4.1mb", "max_in_bytes": 4325376, "peak_max": "4.1mb", "peak_max_in_bytes": 4325376, "peak_used": "2mb", "peak_used_in_bytes": 2162688, "used": "19.4kb", "used_in_bytes": 19904}}}, "threads": {"count": 47, "peak_count": 48}, "timestamp": 1388945167520, "uptime": "23.4m", "uptime_in_millis": 1408987}, "name": "Space Turnip", "network": {"tcp": {"active_opens": 14551, "attempt_fails": 46, "curr_estab": 42, "estab_resets": 124, "in_errs": 82, "in_segs": 3108679, "out_rsts": -1, "out_segs": 2858839, "passive_opens": 289, "retrans_segs": 1272}}, "os": {"cpu": {"idle": 89, "stolen": 0, "sys": 2, "user": 7}, "load_average": [1.7255859375, 1.45556640625, 1.43994140625], "mem": {"actual_free": "1.6gb", "actual_free_in_bytes": 1718706176, "actual_used": "2.3gb", "actual_used_in_bytes": 2576261120, "free": "262.6mb", "free_in_bytes": 275419136, "free_percent": 40, "used": "3.7gb", "used_in_bytes": 4019548160, "used_percent": 59}, "swap": {"free": "1gb", "free_in_bytes": 1073741824, "used": "0b", "used_in_bytes": 0}, "timestamp": 1388945167520, "uptime": "3.8m", "uptime_in_millis": 228110}, "process": {"cpu": {"percent": 0, "sys": "13s", "sys_in_millis": 13058, "total": "1.7m", "total_in_millis": 102484, "user": "1.4m", "user_in_millis": 89426}, "mem": {"resident": "283.2mb", "resident_in_bytes": 296976384, "share": "-1b", "share_in_bytes": -1, "total_virtual": "3.6gb", "total_virtual_in_bytes": 3886198784}, "open_file_descriptors": 134, "timestamp": 1388945167520}, "thread_pool": {"bulk": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "flush": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "generic": {"active": 0, "completed": 252, "largest": 3, "queue": 0, "rejected": 0, "threads": 3}, "get": {"active": 0, "completed": 6, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "index": {"active": 0, "completed": 147, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "management": {"active": 1, "completed": 218, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "merge": {"active": 0, "completed": 36, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "optimize": {"active": 0, "completed": 31, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "percolate": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "refresh": {"active": 0, "completed": 36, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "search": {"active": 0, "completed": 9300, "largest": 6, "queue": 0, "rejected": 0, "threads": 6}, "snapshot": {"active": 0, "completed": 20, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "suggest": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "warmer": {"active": 0, "completed": 74, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}}, "timestamp": 1388945167520, "transport": {"rx_count": 0, "rx_size": "0b", "rx_size_in_bytes": 0, "server_open": 13, "tx_count": 0, "tx_size": "0b", "tx_size_in_bytes": 0}, "transport_address": "inet[/192.168.1.100:9300]"}}}, "meta": {"benchmark_id": "3e878942", "observation_id": "e8cd3f18", "observation_sequence_no": 1, "observation_start": "2014-01-05T18:06:06Z", "observation_stop": "2014-01-05T18:06:07Z", "t_total": "0.02m", "t_total_in_millis": 1178}, "segments": {"num_committed_segments": 1, "num_search_segments": 1, "segments": null, "t_optimize": "0.11s", "t_optimize_in_millis": 109}, "stats": {"docs": {"count": 6, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "26.4kb", "memory_size_in_bytes": 27086}, "dates.date_published": {"memory_size": "48b", "memory_size_in_bytes": 48}}, "memory_size": "26.4kb", "memory_size_in_bytes": 27134}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "32ms", "index_time_in_millis": 32, "index_total": 6}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "276ms", "fetch_time_in_millis": 276, "fetch_total": 300, "groups": {"match_description": {"client_time": "0.34s", "client_time_in_millis": 341, "client_total": 100, "fetch_current": 0, "fetch_time": "84ms", "fetch_time_in_millis": 84, "fetch_total": 100, "query_current": 0, "query_time": "18ms", "query_time_in_millis": 18, "query_total": 100}, "match_description_facet_date_histogram": {"client_time": "0.37s", "client_time_in_millis": 374, "client_total": 100, "fetch_current": 0, "fetch_time": "102ms", "fetch_time_in_millis": 102, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}, "match_description_sorted_abstract": {"client_time": "0.35s", "client_time_in_millis": 352, "client_total": 100, "fetch_current": 0, "fetch_time": "89ms", "fetch_time_in_millis": 89, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "62ms", "query_time_in_millis": 62, "query_total": 300}, "store": {"size": "149.5kb", "size_in_bytes": 153169, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1388945166000]},{"_index":"esbench_stats","_type":"obs","_id":"317405e7","_score":null, "_source" : {"cluster": {"cluster_name": "elasticsearch", "nodes": {"TqybOUCJRPOvLdshFYQqtg": {"fs": {"data": [{"available": "58gb", "available_in_bytes": 62311964672, "dev": "/dev/disk1s2", "disk_read_size": "4.3gb", "disk_read_size_in_bytes": 4703711744, "disk_reads": 248828, "disk_write_size": "15gb", "disk_write_size_in_bytes": 16137129984, "disk_writes": 468640, "free": "58gb", "free_in_bytes": 62311964672, "mount": "/Users/mkocikowski", "path": "/Users/mkocikowski/dev/elasticsearch-0.90.6/data/elasticsearch/nodes/0", "total": "465.1gb", "total_in_bytes": 499430719488}], "timestamp": 1388945169526}, "hostname": "MK.local", "http": {"current_open": 2, "total_opened": 10}, "indices": {"completion": {"size": "0b", "size_in_bytes": 0}, "docs": {"count": 12, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "52.6kb", "memory_size_in_bytes": 53952}, "dates.date_published": {"memory_size": "96b", "memory_size_in_bytes": 96}}, "memory_size": "52.7kb", "memory_size_in_bytes": 54048}, "filter_cache": {"evictions": 0, "memory_size": "48b", "memory_size_in_bytes": 48}, "flush": {"total": 0, "total_time": "0s", "total_time_in_millis": 0}, "get": {"current": 0, "exists_time": "1ms", "exists_time_in_millis": 1, "exists_total": 6, "get_time": "1ms", "missing_time": "0s", "missing_time_in_millis": 0, "missing_total": 0, "time_in_millis": 1, "total": 6}, "id_cache": {"memory_size": "0b", "memory_size_in_bytes": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "2.6s", "index_time_in_millis": 2639, "index_total": 153}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 6, "total_docs": 149, "total_size": "8.6mb", "total_size_in_bytes": 9066343, "total_time": "3s", "total_time_in_millis": 3053}, "refresh": {"total": 44, "total_time": "8s", "total_time_in_millis": 8045}, "search": {"fetch_current": 0, "fetch_time": "29.1s", "fetch_time_in_millis": 29168, "fetch_total": 9600, "open_contexts": 0, "query_current": 0, "query_time": "8.7s", "query_time_in_millis": 8741, "query_total": 9600}, "store": {"size": "523.3kb", "size_in_bytes": 535948, "throttle_time": "0s", "throttle_time_in_millis": 0}, "warmer": {"current": 0, "total": 5, "total_time": "2ms", "total_time_in_millis": 2}}, "jvm": {"gc": {"collection_count": 1206, "collection_time": "4s", "collection_time_in_millis": 4014, "collectors": {"ConcurrentMarkSweep": {"collection_count": 0, "collection_time": "0s", "collection_time_in_millis": 0}, "ParNew": {"collection_count": 1206, "collection_time": "4s", "collection_time_in_millis": 4014}}}, "mem": {"heap_committed": "253.9mb", "heap_committed_in_bytes": 266272768, "heap_used": "137.2mb", "heap_used_in_bytes": 143911208, "non_heap_committed": "46.2mb", "non_heap_committed_in_bytes": 48500736, "non_heap_used": "46mb", "non_heap_used_in_bytes": 48268352, "pools": {"CMS Old Gen": {"max": "982.4mb", "max_in_bytes": 1030160384, "peak_max": "982.4mb", "peak_max_in_bytes": 1030160384, "peak_used": "133.5mb", "peak_used_in_bytes": 140014968, "used": "133.5mb", "used_in_bytes": 140014968}, "CMS Perm Gen": {"max": "82mb", "max_in_bytes": 85983232, "peak_max": "82mb", "peak_max_in_bytes": 85983232, "peak_used": "41.2mb", "peak_used_in_bytes": 43265664, "used": "41.2mb", "used_in_bytes": 43265664}, "Code Cache": {"max": "48mb", "max_in_bytes": 50331648, "peak_max": "48mb", "peak_max_in_bytes": 50331648, "peak_used": "4.7mb", "peak_used_in_bytes": 5011328, "used": "4.7mb", "used_in_bytes": 5002688}, "Par Eden Space": {"max": "33.3mb", "max_in_bytes": 34930688, "peak_max": "33.3mb", "peak_max_in_bytes": 34930688, "peak_used": "16.6mb", "peak_used_in_bytes": 17432576, "used": "3.5mb", "used_in_bytes": 3768768}, "Par Survivor Space": {"max": "4.1mb", "max_in_bytes": 4325376, "peak_max": "4.1mb", "peak_max_in_bytes": 4325376, "peak_used": "2mb", "peak_used_in_bytes": 2162688, "used": "124.4kb", "used_in_bytes": 127472}}}, "threads": {"count": 47, "peak_count": 48}, "timestamp": 1388945169525, "uptime": "23.5m", "uptime_in_millis": 1410992}, "name": "Space Turnip", "network": {"tcp": {"active_opens": 14551, "attempt_fails": 46, "curr_estab": 42, "estab_resets": 124, "in_errs": 82, "in_segs": 3108679, "out_rsts": -1, "out_segs": 2858839, "passive_opens": 289, "retrans_segs": 1272}}, "os": {"cpu": {"idle": 33, "stolen": 0, "sys": 20, "user": 45}, "load_average": [1.7255859375, 1.45556640625, 1.43994140625], "mem": {"actual_free": "1.5gb", "actual_free_in_bytes": 1712320512, "actual_used": "2.4gb", "actual_used_in_bytes": 2582646784, "free": "256.2mb", "free_in_bytes": 268648448, "free_percent": 39, "used": "3.7gb", "used_in_bytes": 4026318848, "used_percent": 60}, "swap": {"free": "1gb", "free_in_bytes": 1073741824, "used": "0b", "used_in_bytes": 0}, "timestamp": 1388945169525, "uptime": "3.8m", "uptime_in_millis": 228112}, "process": {"cpu": {"percent": 89, "sys": "13.4s", "sys_in_millis": 13476, "total": "1.7m", "total_in_millis": 104279, "user": "1.5m", "user_in_millis": 90803}, "mem": {"resident": "287.8mb", "resident_in_bytes": 301838336, "share": "-1b", "share_in_bytes": -1, "total_virtual": "3.6gb", "total_virtual_in_bytes": 3886198784}, "open_file_descriptors": 136, "timestamp": 1388945169525}, "thread_pool": {"bulk": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "flush": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "generic": {"active": 0, "completed": 253, "largest": 3, "queue": 0, "rejected": 0, "threads": 3}, "get": {"active": 0, "completed": 6, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "index": {"active": 0, "completed": 153, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "management": {"active": 1, "completed": 222, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "merge": {"active": 0, "completed": 37, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "optimize": {"active": 0, "completed": 32, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "percolate": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "refresh": {"active": 0, "completed": 37, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "search": {"active": 0, "completed": 9600, "largest": 6, "queue": 0, "rejected": 0, "threads": 6}, "snapshot": {"active": 0, "completed": 20, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "suggest": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "warmer": {"active": 0, "completed": 76, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}}, "timestamp": 1388945169524, "transport": {"rx_count": 0, "rx_size": "0b", "rx_size_in_bytes": 0, "server_open": 13, "tx_count": 0, "tx_size": "0b", "tx_size_in_bytes": 0}, "transport_address": "inet[/192.168.1.100:9300]"}}}, "meta": {"benchmark_id": "3e878942", "observation_id": "317405e7", "observation_sequence_no": 2, "observation_start": "2014-01-05T18:06:07Z", "observation_stop": "2014-01-05T18:06:09Z", "t_total": "0.03m", "t_total_in_millis": 1890}, "segments": {"num_committed_segments": 2, "num_search_segments": 2, "segments": null, "t_optimize": "0.15s", "t_optimize_in_millis": 145}, "stats": {"docs": {"count": 11, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "52.6kb", "memory_size_in_bytes": 53952}, "dates.date_published": {"memory_size": "96b", "memory_size_in_bytes": 96}}, "memory_size": "52.7kb", "memory_size_in_bytes": 54048}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "70ms", "index_time_in_millis": 70, "index_total": 11}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "894ms", "fetch_time_in_millis": 894, "fetch_total": 600, "groups": {"match_description": {"client_time": "0.56s", "client_time_in_millis": 555, "client_total": 100, "fetch_current": 0, "fetch_time": "199ms", "fetch_time_in_millis": 199, "fetch_total": 100, "query_current": 0, "query_time": "34ms", "query_time_in_millis": 34, "query_total": 100}, "match_description_facet_date_histogram": {"client_time": "0.61s", "client_time_in_millis": 612, "client_total": 100, "fetch_current": 0, "fetch_time": "222ms", "fetch_time_in_millis": 222, "fetch_total": 100, "query_current": 0, "query_time": "37ms", "query_time_in_millis": 37, "query_total": 100}, "match_description_sorted_abstract": {"client_time": "0.58s", "client_time_in_millis": 575, "client_total": 100, "fetch_current": 0, "fetch_time": "196ms", "fetch_time_in_millis": 196, "fetch_total": 100, "query_current": 0, "query_time": "40ms", "query_time_in_millis": 40, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "174ms", "query_time_in_millis": 174, "query_total": 600}, "store": {"size": "293.9kb", "size_in_bytes": 300998, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1388945167000]}]}}',
            data=r"""{"took":1,"timed_out":false,"_shards":{"total":1,"successful":1,"failed":0},"hits":{"total":10,"max_score":null,"hits":[{"_index":"esbench_stats","_type":"obs","_id":"e8cd3f18","_score":null, "_source" : {"cluster": {"cluster_name": "elasticsearch", "nodes": {"TqybOUCJRPOvLdshFYQqtg": {"fs": {"data": [{"available": "58gb", "available_in_bytes": 62311968768, "dev": "/dev/disk1s2", "disk_read_size": "4.3gb", "disk_read_size_in_bytes": 4703711744, "disk_reads": 248828, "disk_write_size": "15gb", "disk_write_size_in_bytes": 16135558656, "disk_writes": 468533, "free": "58gb", "free_in_bytes": 62311968768, "mount": "/Users/mkocikowski", "path": "/Users/mkocikowski/dev/elasticsearch-0.90.6/data/elasticsearch/nodes/0", "total": "465.1gb", "total_in_bytes": 499430719488}], "timestamp": 1388945167521}, "hostname": "MK.local", "http": {"current_open": 2, "total_opened": 10}, "indices": {"completion": {"size": "0b", "size_in_bytes": 0}, "docs": {"count": 6, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "26.4kb", "memory_size_in_bytes": 27086}, "dates.date_published": {"memory_size": "48b", "memory_size_in_bytes": 48}}, "memory_size": "26.4kb", "memory_size_in_bytes": 27134}, "filter_cache": {"evictions": 0, "memory_size": "24b", "memory_size_in_bytes": 24}, "flush": {"total": 0, "total_time": "0s", "total_time_in_millis": 0}, "get": {"current": 0, "exists_time": "1ms", "exists_time_in_millis": 1, "exists_total": 6, "get_time": "1ms", "missing_time": "0s", "missing_time_in_millis": 0, "missing_total": 0, "time_in_millis": 1, "total": 6}, "id_cache": {"memory_size": "0b", "memory_size_in_bytes": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "2.5s", "index_time_in_millis": 2558, "index_total": 147}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 6, "total_docs": 149, "total_size": "8.6mb", "total_size_in_bytes": 9066343, "total_time": "3s", "total_time_in_millis": 3053}, "refresh": {"total": 43, "total_time": "7.8s", "total_time_in_millis": 7826}, "search": {"fetch_current": 0, "fetch_time": "28.5s", "fetch_time_in_millis": 28550, "fetch_total": 9300, "open_contexts": 0, "query_current": 0, "query_time": "8.6s", "query_time_in_millis": 8629, "query_total": 9300}, "store": {"size": "149.6kb", "size_in_bytes": 153268, "throttle_time": "0s", "throttle_time_in_millis": 0}, "warmer": {"current": 0, "total": 3, "total_time": "0s", "total_time_in_millis": 0}}, "jvm": {"gc": {"collection_count": 1179, "collection_time": "3.9s", "collection_time_in_millis": 3911, "collectors": {"ConcurrentMarkSweep": {"collection_count": 0, "collection_time": "0s", "collection_time_in_millis": 0}, "ParNew": {"collection_count": 1179, "collection_time": "3.9s", "collection_time_in_millis": 3911}}}, "mem": {"heap_committed": "253.9mb", "heap_committed_in_bytes": 266272768, "heap_used": "137.8mb", "heap_used_in_bytes": 144510592, "non_heap_committed": "46.2mb", "non_heap_committed_in_bytes": 48500736, "non_heap_used": "46mb", "non_heap_used_in_bytes": 48261344, "pools": {"CMS Old Gen": {"max": "982.4mb", "max_in_bytes": 1030160384, "peak_max": "982.4mb", "peak_max_in_bytes": 1030160384, "peak_used": "128.7mb", "peak_used_in_bytes": 135035472, "used": "128.7mb", "used_in_bytes": 135035472}, "CMS Perm Gen": {"max": "82mb", "max_in_bytes": 85983232, "peak_max": "82mb", "peak_max_in_bytes": 85983232, "peak_used": "41.2mb", "peak_used_in_bytes": 43263328, "used": "41.2mb", "used_in_bytes": 43263328}, "Code Cache": {"max": "48mb", "max_in_bytes": 50331648, "peak_max": "48mb", "peak_max_in_bytes": 50331648, "peak_used": "4.7mb", "peak_used_in_bytes": 5008832, "used": "4.7mb", "used_in_bytes": 4998016}, "Par Eden Space": {"max": "33.3mb", "max_in_bytes": 34930688, "peak_max": "33.3mb", "peak_max_in_bytes": 34930688, "peak_used": "16.6mb", "peak_used_in_bytes": 17432576, "used": "9mb", "used_in_bytes": 9455216}, "Par Survivor Space": {"max": "4.1mb", "max_in_bytes": 4325376, "peak_max": "4.1mb", "peak_max_in_bytes": 4325376, "peak_used": "2mb", "peak_used_in_bytes": 2162688, "used": "19.4kb", "used_in_bytes": 19904}}}, "threads": {"count": 47, "peak_count": 48}, "timestamp": 1388945167520, "uptime": "23.4m", "uptime_in_millis": 1408987}, "name": "Space Turnip", "network": {"tcp": {"active_opens": 14551, "attempt_fails": 46, "curr_estab": 42, "estab_resets": 124, "in_errs": 82, "in_segs": 3108679, "out_rsts": -1, "out_segs": 2858839, "passive_opens": 289, "retrans_segs": 1272}}, "os": {"cpu": {"idle": 89, "stolen": 0, "sys": 2, "user": 7}, "load_average": [1.7255859375, 1.45556640625, 1.43994140625], "mem": {"actual_free": "1.6gb", "actual_free_in_bytes": 1718706176, "actual_used": "2.3gb", "actual_used_in_bytes": 2576261120, "free": "262.6mb", "free_in_bytes": 275419136, "free_percent": 40, "used": "3.7gb", "used_in_bytes": 4019548160, "used_percent": 59}, "swap": {"free": "1gb", "free_in_bytes": 1073741824, "used": "0b", "used_in_bytes": 0}, "timestamp": 1388945167520, "uptime": "3.8m", "uptime_in_millis": 228110}, "process": {"cpu": {"percent": 0, "sys": "13s", "sys_in_millis": 13058, "total": "1.7m", "total_in_millis": 102484, "user": "1.4m", "user_in_millis": 89426}, "mem": {"resident": "283.2mb", "resident_in_bytes": 296976384, "share": "-1b", "share_in_bytes": -1, "total_virtual": "3.6gb", "total_virtual_in_bytes": 3886198784}, "open_file_descriptors": 134, "timestamp": 1388945167520}, "thread_pool": {"bulk": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "flush": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "generic": {"active": 0, "completed": 252, "largest": 3, "queue": 0, "rejected": 0, "threads": 3}, "get": {"active": 0, "completed": 6, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "index": {"active": 0, "completed": 147, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "management": {"active": 1, "completed": 218, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "merge": {"active": 0, "completed": 36, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "optimize": {"active": 0, "completed": 31, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "percolate": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "refresh": {"active": 0, "completed": 36, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "search": {"active": 0, "completed": 9300, "largest": 6, "queue": 0, "rejected": 0, "threads": 6}, "snapshot": {"active": 0, "completed": 20, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "suggest": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "warmer": {"active": 0, "completed": 74, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}}, "timestamp": 1388945167520, "transport": {"rx_count": 0, "rx_size": "0b", "rx_size_in_bytes": 0, "server_open": 13, "tx_count": 0, "tx_size": "0b", "tx_size_in_bytes": 0}, "transport_address": "inet[/192.168.1.100:9300]"}}}, "meta": {"benchmark_id": "3e878942", "observation_id": "e8cd3f18", "observation_sequence_no": 1, "observation_start": "2014-01-05T18:06:06Z", "observation_stop": "2014-01-05T18:06:07Z", "t_total": "0.02m", "t_total_in_millis": 1178}, "segments": {"num_committed_segments": 1, "num_search_segments": 1, "segments": null, "t_optimize": "0.11s", "t_optimize_in_millis": 109}, "stats": {"docs": {"count": 6, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "26.4kb", "memory_size_in_bytes": 27086}, "dates.date_published": {"memory_size": "48b", "memory_size_in_bytes": 48}}, "memory_size": "26.4kb", "memory_size_in_bytes": 27134}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "32ms", "index_time_in_millis": 32, "index_total": 6}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "276ms", "fetch_time_in_millis": 276, "fetch_total": 300, "groups": {"match_description": {"client_time": "0.34s", "client_time_in_millis": 341, "client_total": 100, "fetch_current": 0, "fetch_time": "84ms", "fetch_time_in_millis": 84, "fetch_total": 100, "query_current": 0, "query_time": "18ms", "query_time_in_millis": 18, "query_total": 100}, "match_description_facet_date_histogram": {"client_time": "0.37s", "client_time_in_millis": 374, "client_total": 100, "fetch_current": 0, "fetch_time": "102ms", "fetch_time_in_millis": 102, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}, "match_description_sorted_abstract": {"client_time": "0.35s", "client_time_in_millis": 352, "client_total": 100, "fetch_current": 0, "fetch_time": "89ms", "fetch_time_in_millis": 89, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "62ms", "query_time_in_millis": 62, "query_total": 300}, "store": {"size": "149.5kb", "size_in_bytes": 153169, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1388945166000]},{"_index":"esbench_stats","_type":"obs","_id":"317405e7","_score":null, "_source" : {"cluster": {"cluster_name": "elasticsearch", "nodes": {"TqybOUCJRPOvLdshFYQqtg": {"fs": {"data": [{"available": "58gb", "available_in_bytes": 62311964672, "dev": "/dev/disk1s2", "disk_read_size": "4.3gb", "disk_read_size_in_bytes": 4703711744, "disk_reads": 248828, "disk_write_size": "15gb", "disk_write_size_in_bytes": 16137129984, "disk_writes": 468640, "free": "58gb", "free_in_bytes": 62311964672, "mount": "/Users/mkocikowski", "path": "/Users/mkocikowski/dev/elasticsearch-0.90.6/data/elasticsearch/nodes/0", "total": "465.1gb", "total_in_bytes": 499430719488}], "timestamp": 1388945169526}, "hostname": "MK.local", "http": {"current_open": 2, "total_opened": 10}, "indices": {"completion": {"size": "0b", "size_in_bytes": 0}, "docs": {"count": 12, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "52.6kb", "memory_size_in_bytes": 53952}, "dates.date_published": {"memory_size": "96b", "memory_size_in_bytes": 96}}, "memory_size": "52.7kb", "memory_size_in_bytes": 54048}, "filter_cache": {"evictions": 0, "memory_size": "48b", "memory_size_in_bytes": 48}, "flush": {"total": 0, "total_time": "0s", "total_time_in_millis": 0}, "get": {"current": 0, "exists_time": "1ms", "exists_time_in_millis": 1, "exists_total": 6, "get_time": "1ms", "missing_time": "0s", "missing_time_in_millis": 0, "missing_total": 0, "time_in_millis": 1, "total": 6}, "id_cache": {"memory_size": "0b", "memory_size_in_bytes": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "2.6s", "index_time_in_millis": 2639, "index_total": 153}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 6, "total_docs": 149, "total_size": "8.6mb", "total_size_in_bytes": 9066343, "total_time": "3s", "total_time_in_millis": 3053}, "refresh": {"total": 44, "total_time": "8s", "total_time_in_millis": 8045}, "search": {"fetch_current": 0, "fetch_time": "29.1s", "fetch_time_in_millis": 29168, "fetch_total": 9600, "open_contexts": 0, "query_current": 0, "query_time": "8.7s", "query_time_in_millis": 8741, "query_total": 9600}, "store": {"size": "523.3kb", "size_in_bytes": 535948, "throttle_time": "0s", "throttle_time_in_millis": 0}, "warmer": {"current": 0, "total": 5, "total_time": "2ms", "total_time_in_millis": 2}}, "jvm": {"gc": {"collection_count": 1206, "collection_time": "4s", "collection_time_in_millis": 4014, "collectors": {"ConcurrentMarkSweep": {"collection_count": 0, "collection_time": "0s", "collection_time_in_millis": 0}, "ParNew": {"collection_count": 1206, "collection_time": "4s", "collection_time_in_millis": 4014}}}, "mem": {"heap_committed": "253.9mb", "heap_committed_in_bytes": 266272768, "heap_used": "137.2mb", "heap_used_in_bytes": 143911208, "non_heap_committed": "46.2mb", "non_heap_committed_in_bytes": 48500736, "non_heap_used": "46mb", "non_heap_used_in_bytes": 48268352, "pools": {"CMS Old Gen": {"max": "982.4mb", "max_in_bytes": 1030160384, "peak_max": "982.4mb", "peak_max_in_bytes": 1030160384, "peak_used": "133.5mb", "peak_used_in_bytes": 140014968, "used": "133.5mb", "used_in_bytes": 140014968}, "CMS Perm Gen": {"max": "82mb", "max_in_bytes": 85983232, "peak_max": "82mb", "peak_max_in_bytes": 85983232, "peak_used": "41.2mb", "peak_used_in_bytes": 43265664, "used": "41.2mb", "used_in_bytes": 43265664}, "Code Cache": {"max": "48mb", "max_in_bytes": 50331648, "peak_max": "48mb", "peak_max_in_bytes": 50331648, "peak_used": "4.7mb", "peak_used_in_bytes": 5011328, "used": "4.7mb", "used_in_bytes": 5002688}, "Par Eden Space": {"max": "33.3mb", "max_in_bytes": 34930688, "peak_max": "33.3mb", "peak_max_in_bytes": 34930688, "peak_used": "16.6mb", "peak_used_in_bytes": 17432576, "used": "3.5mb", "used_in_bytes": 3768768}, "Par Survivor Space": {"max": "4.1mb", "max_in_bytes": 4325376, "peak_max": "4.1mb", "peak_max_in_bytes": 4325376, "peak_used": "2mb", "peak_used_in_bytes": 2162688, "used": "124.4kb", "used_in_bytes": 127472}}}, "threads": {"count": 47, "peak_count": 48}, "timestamp": 1388945169525, "uptime": "23.5m", "uptime_in_millis": 1410992}, "name": "Space Turnip", "network": {"tcp": {"active_opens": 14551, "attempt_fails": 46, "curr_estab": 42, "estab_resets": 124, "in_errs": 82, "in_segs": 3108679, "out_rsts": -1, "out_segs": 2858839, "passive_opens": 289, "retrans_segs": 1272}}, "os": {"cpu": {"idle": 33, "stolen": 0, "sys": 20, "user": 45}, "load_average": [1.7255859375, 1.45556640625, 1.43994140625], "mem": {"actual_free": "1.5gb", "actual_free_in_bytes": 1712320512, "actual_used": "2.4gb", "actual_used_in_bytes": 2582646784, "free": "256.2mb", "free_in_bytes": 268648448, "free_percent": 39, "used": "3.7gb", "used_in_bytes": 4026318848, "used_percent": 60}, "swap": {"free": "1gb", "free_in_bytes": 1073741824, "used": "0b", "used_in_bytes": 0}, "timestamp": 1388945169525, "uptime": "3.8m", "uptime_in_millis": 228112}, "process": {"cpu": {"percent": 89, "sys": "13.4s", "sys_in_millis": 13476, "total": "1.7m", "total_in_millis": 104279, "user": "1.5m", "user_in_millis": 90803}, "mem": {"resident": "287.8mb", "resident_in_bytes": 301838336, "share": "-1b", "share_in_bytes": -1, "total_virtual": "3.6gb", "total_virtual_in_bytes": 3886198784}, "open_file_descriptors": 136, "timestamp": 1388945169525}, "thread_pool": {"bulk": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "flush": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "generic": {"active": 0, "completed": 253, "largest": 3, "queue": 0, "rejected": 0, "threads": 3}, "get": {"active": 0, "completed": 6, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "index": {"active": 0, "completed": 153, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "management": {"active": 1, "completed": 222, "largest": 2, "queue": 0, "rejected": 0, "threads": 2}, "merge": {"active": 0, "completed": 37, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "optimize": {"active": 0, "completed": 32, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "percolate": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "refresh": {"active": 0, "completed": 37, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "search": {"active": 0, "completed": 9600, "largest": 6, "queue": 0, "rejected": 0, "threads": 6}, "snapshot": {"active": 0, "completed": 20, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}, "suggest": {"active": 0, "completed": 0, "largest": 0, "queue": 0, "rejected": 0, "threads": 0}, "warmer": {"active": 0, "completed": 76, "largest": 1, "queue": 0, "rejected": 0, "threads": 1}}, "timestamp": 1388945169524, "transport": {"rx_count": 0, "rx_size": "0b", "rx_size_in_bytes": 0, "server_open": 13, "tx_count": 0, "tx_size": "0b", "tx_size_in_bytes": 0}, "transport_address": "inet[/192.168.1.100:9300]"}}}, "meta": {"benchmark_id": "3e878942", "observation_id": "317405e7", "observation_sequence_no": 2, "observation_start": "2014-01-05T18:06:07Z", "observation_stop": "2014-01-05T18:06:09Z", "t_total": "0.03m", "t_total_in_millis": 1890}, "segments": {"num_committed_segments": 2, "num_search_segments": 2, "segments": null, "t_optimize": "0.15s", "t_optimize_in_millis": 145}, "stats": {"docs": {"count": 11, "deleted": 0}, "fielddata": {"evictions": 0, "fields": {"abstract": {"memory_size": "52.6kb", "memory_size_in_bytes": 53952}, "dates.date_published": {"memory_size": "96b", "memory_size_in_bytes": 96}}, "memory_size": "52.7kb", "memory_size_in_bytes": 54048}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "70ms", "index_time_in_millis": 70, "index_total": 11}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "894ms", "fetch_time_in_millis": 894, "fetch_total": 600, "groups": {"match_description": {"client_time": "0.56s", "client_time_in_millis": 555, "client_total": 100, "fetch_current": 0, "fetch_time": "199ms", "fetch_time_in_millis": 199, "fetch_total": 100, "query_current": 0, "query_time": "34ms", "query_time_in_millis": 34, "query_total": 100}, "match_description_facet_date_histogram": {"client_time": "0.61s", "client_time_in_millis": 612, "client_total": 100, "fetch_current": 0, "fetch_time": "222ms", "fetch_time_in_millis": 222, "fetch_total": 100, "query_current": 0, "query_time": "37ms", "query_time_in_millis": 37, "query_total": 100}, "match_description_sorted_abstract": {"client_time": "0.58s", "client_time_in_millis": 575, "client_total": 100, "fetch_current": 0, "fetch_time": "196ms", "fetch_time_in_millis": 196, "fetch_total": 100, "query_current": 0, "query_time": "40ms", "query_time_in_millis": 40, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "174ms", "query_time_in_millis": 174, "query_total": 600}, "store": {"size": "293.9kb", "size_in_bytes": 300998, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1388945167000]}]}}""",
            curl='curl "http://localhost:9200/esbench_stats/obs/_search?q=meta.benchmark_id:3e878942&sort=meta.observation_start:asc&size=2"'
        )

    def tearDown(self):
        esbench.analyze._get_benchmarks = self.tmp_get_benchmarks
        esbench.analyze._get_observations = self.tmp_get_observations


    def test_benchmarks(self):
        self.assertEqual(1, len(list(esbench.analyze._benchmarks(resp=esbench.analyze._get_benchmarks()))))
        self.assertEqual(1, len(list(esbench.analyze._benchmarks(resp=esbench.analyze._get_benchmarks(), benchmark_ids=['3e878942', 'foo']))))
        self.assertEqual(0, len(list(esbench.analyze._benchmarks(resp=esbench.analyze._get_benchmarks(), benchmark_ids=['foo']))))

        # this is the response you get when there are no benchmarks
        resp = esbench.api.ApiResponse(status=200, reason='OK', data='{"took":1,"timed_out":false}', curl='curl -XGET http://localhost:9200/stats/bench/_search?sort=benchmark_start:asc&size=100')
        self.assertEqual(0, len(list(esbench.analyze._benchmarks(resp=resp))))

        r = esbench.api.ApiResponse(
            status=200,
            reason='OK',
            data = '{"hits": {"hits": [{"_id": "foo"}, {"_id": "bar"}, {"_id": "baz"}]}}',
            curl='curl "http://localhost:9200/esbench_stats/bench/_search?sort=benchmark_start:asc&size=100"',
        )
        self.assertEqual([{u'_id': u'foo'}, {u'_id': u'bar'}, {u'_id': u'baz'}], list(esbench.analyze._benchmarks(resp=r)))
        self.assertEqual([{u'_id': u'foo'}, {u'_id': u'bar'}, {u'_id': u'baz'}], list(esbench.analyze._benchmarks(resp=r, benchmark_ids=['all'])))
        self.assertEqual([{u'_id': u'foo'}, {u'_id': u'baz'}], list(esbench.analyze._benchmarks(resp=r, benchmark_ids=['first', 'last'])))
        self.assertEqual([{u'_id': u'bar'}, {u'_id': u'baz'}], list(esbench.analyze._benchmarks(resp=r, benchmark_ids=['1', 'last'])))
        self.assertEqual([{u'_id': u'baz'}, {u'_id': u'foo'}, {u'_id': u'bar'}, {u'_id': u'baz'}], list(esbench.analyze._benchmarks(resp=r, benchmark_ids=['-1', 'all'])))


    def test_observations(self):
        resp = esbench.api.ApiResponse(status=200, reason='OK', data='{"took":5,"timed_out":false,"_shards":{"total":5,"successful":5,"failed":0},"hits":{"total":10,"max_score":null,"hits":[{"_index":"stats","_type":"obs","_id":"c5ac7b40","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "c5ac7b40", "observation_sequence_no": 1, "observation_start": "2013-11-26T17:58:12Z", "observation_stop": "2013-11-26T17:58:13Z"}, "segments": {"num_committed_segments": 1, "num_search_segments": 1, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}}, "t_optimize": "0.17s", "t_optimize_in_millis": 165}, "stats": {"docs": {"count": 10, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "234ms", "index_time_in_millis": 234, "index_total": 10}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "461ms", "fetch_time_in_millis": 461, "fetch_total": 500, "groups": {"match": {"client_time": "0.34s", "client_time_in_millis": 342, "client_total": 100, "fetch_current": 0, "fetch_time": "113ms", "fetch_time_in_millis": 113, "fetch_total": 100, "query_current": 0, "query_time": "39ms", "query_time_in_millis": 39, "query_total": 100}, "match_abs": {"client_time": "0.29s", "client_time_in_millis": 294, "client_total": 100, "fetch_current": 0, "fetch_time": "86ms", "fetch_time_in_millis": 86, "fetch_total": 100, "query_current": 0, "query_time": "45ms", "query_time_in_millis": 45, "query_total": 100}, "match_no_rnd": {"client_time": "0.45s", "client_time_in_millis": 448, "client_total": 100, "fetch_current": 0, "fetch_time": "156ms", "fetch_time_in_millis": 156, "fetch_total": 100, "query_current": 0, "query_time": "58ms", "query_time_in_millis": 58, "query_total": 100}, "match_srt": {"client_time": "0.32s", "client_time_in_millis": 315, "client_total": 100, "fetch_current": 0, "fetch_time": "102ms", "fetch_time_in_millis": 102, "fetch_total": 100, "query_current": 0, "query_time": "38ms", "query_time_in_millis": 38, "query_total": 100}, "mlt": {"client_time": "0.14s", "client_time_in_millis": 136, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "11ms", "query_time_in_millis": 11, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "193ms", "query_time_in_millis": 193, "query_total": 500}, "store": {"size": "237.5kb", "size_in_bytes": 243242, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488692000]},{"_index":"stats","_type":"obs","_id":"2f74c0e9","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "2f74c0e9", "observation_sequence_no": 2, "observation_start": "2013-11-26T17:58:13Z", "observation_stop": "2013-11-26T17:58:16Z"}, "segments": {"num_committed_segments": 2, "num_search_segments": 2, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}}, "t_optimize": "0.07s", "t_optimize_in_millis": 66}, "stats": {"docs": {"count": 20, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "271ms", "index_time_in_millis": 271, "index_total": 20}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "1.2s", "fetch_time_in_millis": 1206, "fetch_total": 1000, "groups": {"match": {"client_time": "0.84s", "client_time_in_millis": 844, "client_total": 100, "fetch_current": 0, "fetch_time": "189ms", "fetch_time_in_millis": 189, "fetch_total": 100, "query_current": 0, "query_time": "52ms", "query_time_in_millis": 52, "query_total": 100}, "match_abs": {"client_time": "0.35s", "client_time_in_millis": 352, "client_total": 100, "fetch_current": 0, "fetch_time": "135ms", "fetch_time_in_millis": 135, "fetch_total": 100, "query_current": 0, "query_time": "60ms", "query_time_in_millis": 60, "query_total": 100}, "match_no_rnd": {"client_time": "0.57s", "client_time_in_millis": 567, "client_total": 100, "fetch_current": 0, "fetch_time": "230ms", "fetch_time_in_millis": 230, "fetch_total": 100, "query_current": 0, "query_time": "70ms", "query_time_in_millis": 70, "query_total": 100}, "match_srt": {"client_time": "0.46s", "client_time_in_millis": 460, "client_total": 100, "fetch_current": 0, "fetch_time": "188ms", "fetch_time_in_millis": 188, "fetch_total": 100, "query_current": 0, "query_time": "49ms", "query_time_in_millis": 49, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 130, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "13ms", "query_time_in_millis": 13, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "439ms", "query_time_in_millis": 439, "query_total": 1000}, "store": {"size": "481.8kb", "size_in_bytes": 493437, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488693000]},{"_index":"stats","_type":"obs","_id":"a00c16c7","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "a00c16c7", "observation_sequence_no": 3, "observation_start": "2013-11-26T17:58:16Z", "observation_stop": "2013-11-26T17:58:18Z"}, "segments": {"num_committed_segments": 3, "num_search_segments": 3, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}}, "t_optimize": "0.05s", "t_optimize_in_millis": 51}, "stats": {"docs": {"count": 30, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "328ms", "index_time_in_millis": 328, "index_total": 30}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "2.1s", "fetch_time_in_millis": 2171, "fetch_total": 1500, "groups": {"match": {"client_time": "0.59s", "client_time_in_millis": 588, "client_total": 100, "fetch_current": 0, "fetch_time": "270ms", "fetch_time_in_millis": 270, "fetch_total": 100, "query_current": 0, "query_time": "51ms", "query_time_in_millis": 51, "query_total": 100}, "match_abs": {"client_time": "0.40s", "client_time_in_millis": 399, "client_total": 100, "fetch_current": 0, "fetch_time": "196ms", "fetch_time_in_millis": 196, "fetch_total": 100, "query_current": 0, "query_time": "54ms", "query_time_in_millis": 54, "query_total": 100}, "match_no_rnd": {"client_time": "0.61s", "client_time_in_millis": 610, "client_total": 100, "fetch_current": 0, "fetch_time": "281ms", "fetch_time_in_millis": 281, "fetch_total": 100, "query_current": 0, "query_time": "60ms", "query_time_in_millis": 60, "query_total": 100}, "match_srt": {"client_time": "0.50s", "client_time_in_millis": 496, "client_total": 100, "fetch_current": 0, "fetch_time": "214ms", "fetch_time_in_millis": 214, "fetch_total": 100, "query_current": 0, "query_time": "50ms", "query_time_in_millis": 50, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 133, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "16ms", "query_time_in_millis": 16, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "672ms", "query_time_in_millis": 672, "query_total": 1500}, "store": {"size": "856.4kb", "size_in_bytes": 877007, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488696000]},{"_index":"stats","_type":"obs","_id":"244e5a42","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "244e5a42", "observation_sequence_no": 4, "observation_start": "2013-11-26T17:58:18Z", "observation_stop": "2013-11-26T17:58:21Z"}, "segments": {"num_committed_segments": 5, "num_search_segments": 5, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}}, "t_optimize": "0.02s", "t_optimize_in_millis": 21}, "stats": {"docs": {"count": 40, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "439ms", "index_time_in_millis": 439, "index_total": 40}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "3.1s", "fetch_time_in_millis": 3151, "fetch_total": 2000, "groups": {"match": {"client_time": "0.63s", "client_time_in_millis": 628, "client_total": 100, "fetch_current": 0, "fetch_time": "290ms", "fetch_time_in_millis": 290, "fetch_total": 100, "query_current": 0, "query_time": "56ms", "query_time_in_millis": 56, "query_total": 100}, "match_abs": {"client_time": "0.36s", "client_time_in_millis": 363, "client_total": 100, "fetch_current": 0, "fetch_time": "184ms", "fetch_time_in_millis": 184, "fetch_total": 100, "query_current": 0, "query_time": "49ms", "query_time_in_millis": 49, "query_total": 100}, "match_no_rnd": {"client_time": "0.66s", "client_time_in_millis": 656, "client_total": 100, "fetch_current": 0, "fetch_time": "303ms", "fetch_time_in_millis": 303, "fetch_total": 100, "query_current": 0, "query_time": "57ms", "query_time_in_millis": 57, "query_total": 100}, "match_srt": {"client_time": "0.48s", "client_time_in_millis": 482, "client_total": 100, "fetch_current": 0, "fetch_time": "198ms", "fetch_time_in_millis": 198, "fetch_total": 100, "query_current": 0, "query_time": "61ms", "query_time_in_millis": 61, "query_total": 100}, "mlt": {"client_time": "0.14s", "client_time_in_millis": 137, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "21ms", "query_time_in_millis": 21, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "918ms", "query_time_in_millis": 918, "query_total": 2000}, "store": {"size": "1.2mb", "size_in_bytes": 1325157, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488698000]},{"_index":"stats","_type":"obs","_id":"c0e74e4d","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "c0e74e4d", "observation_sequence_no": 5, "observation_start": "2013-11-26T17:58:21Z", "observation_stop": "2013-11-26T17:58:23Z"}, "segments": {"num_committed_segments": 6, "num_search_segments": 6, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}, "_5": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 5, "num_docs": 10, "search": true, "size": "350kb", "size_in_bytes": 358422, "version": "4.5.1"}}, "t_optimize": "0.04s", "t_optimize_in_millis": 39}, "stats": {"docs": {"count": 50, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "504ms", "index_time_in_millis": 504, "index_total": 50}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "4s", "fetch_time_in_millis": 4067, "fetch_total": 2500, "groups": {"match": {"client_time": "0.60s", "client_time_in_millis": 602, "client_total": 100, "fetch_current": 0, "fetch_time": "266ms", "fetch_time_in_millis": 266, "fetch_total": 100, "query_current": 0, "query_time": "60ms", "query_time_in_millis": 60, "query_total": 100}, "match_abs": {"client_time": "0.35s", "client_time_in_millis": 352, "client_total": 100, "fetch_current": 0, "fetch_time": "175ms", "fetch_time_in_millis": 175, "fetch_total": 100, "query_current": 0, "query_time": "51ms", "query_time_in_millis": 51, "query_total": 100}, "match_no_rnd": {"client_time": "0.60s", "client_time_in_millis": 601, "client_total": 100, "fetch_current": 0, "fetch_time": "267ms", "fetch_time_in_millis": 267, "fetch_total": 100, "query_current": 0, "query_time": "58ms", "query_time_in_millis": 58, "query_total": 100}, "match_srt": {"client_time": "0.49s", "client_time_in_millis": 492, "client_total": 100, "fetch_current": 0, "fetch_time": "205ms", "fetch_time_in_millis": 205, "fetch_total": 100, "query_current": 0, "query_time": "57ms", "query_time_in_millis": 57, "query_total": 100}, "mlt": {"client_time": "0.14s", "client_time_in_millis": 139, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "23ms", "query_time_in_millis": 23, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.1s", "query_time_in_millis": 1169, "query_total": 2500}, "store": {"size": "1.6mb", "size_in_bytes": 1683644, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488701000]},{"_index":"stats","_type":"obs","_id":"6ea3adfe","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "6ea3adfe", "observation_sequence_no": 6, "observation_start": "2013-11-26T17:58:23Z", "observation_stop": "2013-11-26T17:58:26Z"}, "segments": {"num_committed_segments": 7, "num_search_segments": 7, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}, "_5": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 5, "num_docs": 10, "search": true, "size": "350kb", "size_in_bytes": 358422, "version": "4.5.1"}, "_6": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 6, "num_docs": 10, "search": true, "size": "388.6kb", "size_in_bytes": 397956, "version": "4.5.1"}}, "t_optimize": "0.03s", "t_optimize_in_millis": 30}, "stats": {"docs": {"count": 60, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "558ms", "index_time_in_millis": 558, "index_total": 60}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "5s", "fetch_time_in_millis": 5092, "fetch_total": 3000, "groups": {"match": {"client_time": "0.68s", "client_time_in_millis": 678, "client_total": 100, "fetch_current": 0, "fetch_time": "307ms", "fetch_time_in_millis": 307, "fetch_total": 100, "query_current": 0, "query_time": "53ms", "query_time_in_millis": 53, "query_total": 100}, "match_abs": {"client_time": "0.38s", "client_time_in_millis": 378, "client_total": 100, "fetch_current": 0, "fetch_time": "205ms", "fetch_time_in_millis": 205, "fetch_total": 100, "query_current": 0, "query_time": "45ms", "query_time_in_millis": 45, "query_total": 100}, "match_no_rnd": {"client_time": "0.68s", "client_time_in_millis": 684, "client_total": 100, "fetch_current": 0, "fetch_time": "311ms", "fetch_time_in_millis": 311, "fetch_total": 100, "query_current": 0, "query_time": "48ms", "query_time_in_millis": 48, "query_total": 100}, "match_srt": {"client_time": "0.48s", "client_time_in_millis": 480, "client_total": 100, "fetch_current": 0, "fetch_time": "198ms", "fetch_time_in_millis": 198, "fetch_total": 100, "query_current": 0, "query_time": "56ms", "query_time_in_millis": 56, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 133, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "19ms", "query_time_in_millis": 19, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.3s", "query_time_in_millis": 1393, "query_total": 3000}, "store": {"size": "1.9mb", "size_in_bytes": 2081666, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488703000]},{"_index":"stats","_type":"obs","_id":"d7030c18","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "d7030c18", "observation_sequence_no": 7, "observation_start": "2013-11-26T17:58:26Z", "observation_stop": "2013-11-26T17:58:28Z"}, "segments": {"num_committed_segments": 8, "num_search_segments": 8, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}, "_5": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 5, "num_docs": 10, "search": true, "size": "350kb", "size_in_bytes": 358422, "version": "4.5.1"}, "_6": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 6, "num_docs": 10, "search": true, "size": "388.6kb", "size_in_bytes": 397956, "version": "4.5.1"}, "_7": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 7, "num_docs": 10, "search": true, "size": "395.3kb", "size_in_bytes": 404797, "version": "4.5.1"}}, "t_optimize": "0.03s", "t_optimize_in_millis": 34}, "stats": {"docs": {"count": 70, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "634ms", "index_time_in_millis": 634, "index_total": 70}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "6.1s", "fetch_time_in_millis": 6122, "fetch_total": 3500, "groups": {"match": {"client_time": "0.68s", "client_time_in_millis": 683, "client_total": 100, "fetch_current": 0, "fetch_time": "298ms", "fetch_time_in_millis": 298, "fetch_total": 100, "query_current": 0, "query_time": "63ms", "query_time_in_millis": 63, "query_total": 100}, "match_abs": {"client_time": "0.38s", "client_time_in_millis": 378, "client_total": 100, "fetch_current": 0, "fetch_time": "203ms", "fetch_time_in_millis": 203, "fetch_total": 100, "query_current": 0, "query_time": "49ms", "query_time_in_millis": 49, "query_total": 100}, "match_no_rnd": {"client_time": "0.67s", "client_time_in_millis": 673, "client_total": 100, "fetch_current": 0, "fetch_time": "312ms", "fetch_time_in_millis": 312, "fetch_total": 100, "query_current": 0, "query_time": "48ms", "query_time_in_millis": 48, "query_total": 100}, "match_srt": {"client_time": "0.48s", "client_time_in_millis": 476, "client_total": 100, "fetch_current": 0, "fetch_time": "214ms", "fetch_time_in_millis": 214, "fetch_total": 100, "query_current": 0, "query_time": "52ms", "query_time_in_millis": 52, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 134, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "20ms", "query_time_in_millis": 20, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.6s", "query_time_in_millis": 1627, "query_total": 3500}, "store": {"size": "2.3mb", "size_in_bytes": 2486530, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488706000]},{"_index":"stats","_type":"obs","_id":"8b615e3f","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "8b615e3f", "observation_sequence_no": 8, "observation_start": "2013-11-26T17:58:28Z", "observation_stop": "2013-11-26T17:58:31Z"}, "segments": {"num_committed_segments": 9, "num_search_segments": 9, "segments": {"_0": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 0, "num_docs": 10, "search": true, "size": "237.3kb", "size_in_bytes": 243076, "version": "4.5.1"}, "_1": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 1, "num_docs": 10, "search": true, "size": "244.2kb", "size_in_bytes": 250129, "version": "4.5.1"}, "_2": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 2, "num_docs": 10, "search": true, "size": "374.5kb", "size_in_bytes": 383503, "version": "4.5.1"}, "_3": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 3, "num_docs": 8, "search": true, "size": "305.1kb", "size_in_bytes": 312523, "version": "4.5.1"}, "_4": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 4, "num_docs": 2, "search": true, "size": "132.3kb", "size_in_bytes": 135496, "version": "4.5.1"}, "_5": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 5, "num_docs": 10, "search": true, "size": "350kb", "size_in_bytes": 358422, "version": "4.5.1"}, "_6": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 6, "num_docs": 10, "search": true, "size": "388.6kb", "size_in_bytes": 397956, "version": "4.5.1"}, "_7": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 7, "num_docs": 10, "search": true, "size": "395.3kb", "size_in_bytes": 404797, "version": "4.5.1"}, "_8": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 8, "num_docs": 10, "search": true, "size": "465.1kb", "size_in_bytes": 476342, "version": "4.5.1"}}, "t_optimize": "0.04s", "t_optimize_in_millis": 35}, "stats": {"docs": {"count": 80, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "693ms", "index_time_in_millis": 693, "index_total": 80}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 0, "total_docs": 0, "total_size": "0b", "total_size_in_bytes": 0, "total_time": "0s", "total_time_in_millis": 0}, "search": {"fetch_current": 0, "fetch_time": "7.1s", "fetch_time_in_millis": 7157, "fetch_total": 4000, "groups": {"match": {"client_time": "0.69s", "client_time_in_millis": 691, "client_total": 100, "fetch_current": 0, "fetch_time": "313ms", "fetch_time_in_millis": 313, "fetch_total": 100, "query_current": 0, "query_time": "62ms", "query_time_in_millis": 62, "query_total": 100}, "match_abs": {"client_time": "0.38s", "client_time_in_millis": 379, "client_total": 100, "fetch_current": 0, "fetch_time": "205ms", "fetch_time_in_millis": 205, "fetch_total": 100, "query_current": 0, "query_time": "51ms", "query_time_in_millis": 51, "query_total": 100}, "match_no_rnd": {"client_time": "0.69s", "client_time_in_millis": 687, "client_total": 100, "fetch_current": 0, "fetch_time": "318ms", "fetch_time_in_millis": 318, "fetch_total": 100, "query_current": 0, "query_time": "55ms", "query_time_in_millis": 55, "query_total": 100}, "match_srt": {"client_time": "0.48s", "client_time_in_millis": 483, "client_total": 100, "fetch_current": 0, "fetch_time": "195ms", "fetch_time_in_millis": 195, "fetch_total": 100, "query_current": 0, "query_time": "69ms", "query_time_in_millis": 69, "query_total": 100}, "mlt": {"client_time": "0.13s", "client_time_in_millis": 131, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.8s", "query_time_in_millis": 1889, "query_total": 4000}, "store": {"size": "2.8mb", "size_in_bytes": 2962937, "throttle_time": "0s", "throttle_time_in_millis": 0}}},"sort":[1385488708000]},{"_index":"stats","_type":"obs","_id":"a0c28e25","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "a0c28e25", "observation_sequence_no": 9, "observation_start": "2013-11-26T17:58:31Z", "observation_stop": "2013-11-26T17:58:34Z"}, "segments": {"num_committed_segments": 1, "num_search_segments": 1, "segments": {"_a": {"committed": true, "compound": false, "deleted_docs": 0, "generation": 10, "num_docs": 90, "search": true, "size": "2.7mb", "size_in_bytes": 2853865, "version": "4.5.1"}}, "t_optimize": "0.46s", "t_optimize_in_millis": 457}, "stats": {"docs": {"count": 90, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "730ms", "index_time_in_millis": 730, "index_total": 90}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 1, "total_docs": 90, "total_size": "3mb", "total_size_in_bytes": 3248531, "total_time": "423ms", "total_time_in_millis": 423}, "search": {"fetch_current": 0, "fetch_time": "8.3s", "fetch_time_in_millis": 8334, "fetch_total": 4500, "groups": {"match": {"client_time": "0.65s", "client_time_in_millis": 654, "client_total": 100, "fetch_current": 0, "fetch_time": "326ms", "fetch_time_in_millis": 326, "fetch_total": 100, "query_current": 0, "query_time": "25ms", "query_time_in_millis": 25, "query_total": 100}, "match_abs": {"client_time": "0.35s", "client_time_in_millis": 351, "client_total": 100, "fetch_current": 0, "fetch_time": "210ms", "fetch_time_in_millis": 210, "fetch_total": 100, "query_current": 0, "query_time": "18ms", "query_time_in_millis": 18, "query_total": 100}, "match_no_rnd": {"client_time": "0.67s", "client_time_in_millis": 668, "client_total": 100, "fetch_current": 0, "fetch_time": "334ms", "fetch_time_in_millis": 334, "fetch_total": 100, "query_current": 0, "query_time": "22ms", "query_time_in_millis": 22, "query_total": 100}, "match_srt": {"client_time": "0.65s", "client_time_in_millis": 646, "client_total": 100, "fetch_current": 0, "fetch_time": "304ms", "fetch_time_in_millis": 304, "fetch_total": 100, "query_current": 0, "query_time": "28ms", "query_time_in_millis": 28, "query_total": 100}, "mlt": {"client_time": "0.11s", "client_time_in_millis": 108, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "7ms", "query_time_in_millis": 7, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "1.9s", "query_time_in_millis": 1993, "query_total": 4500}, "store": {"size": "2.7mb", "size_in_bytes": 2854630, "throttle_time": "120.3ms", "throttle_time_in_millis": 120}}},"sort":[1385488711000]},{"_index":"stats","_type":"obs","_id":"6d183b4c","_score":null, "_source" : {"meta": {"benchmark_id": "315fd1aa", "observation_id": "6d183b4c", "observation_sequence_no": 10, "observation_start": "2013-11-26T17:58:34Z", "observation_stop": "2013-11-26T17:58:36Z"}, "segments": {"num_committed_segments": 2, "num_search_segments": 2, "segments": {"_a": {"committed": true, "compound": false, "deleted_docs": 0, "generation": 10, "num_docs": 90, "search": true, "size": "2.7mb", "size_in_bytes": 2853865, "version": "4.5.1"}, "_b": {"committed": true, "compound": true, "deleted_docs": 0, "generation": 11, "num_docs": 10, "search": true, "size": "231.2kb", "size_in_bytes": 236834, "version": "4.5.1"}}, "t_optimize": "0.03s", "t_optimize_in_millis": 26}, "stats": {"docs": {"count": 100, "deleted": 0}, "indexing": {"delete_current": 0, "delete_time": "0s", "delete_time_in_millis": 0, "delete_total": 0, "index_current": 0, "index_time": "765ms", "index_time_in_millis": 765, "index_total": 100}, "merges": {"current": 0, "current_docs": 0, "current_size": "0b", "current_size_in_bytes": 0, "total": 1, "total_docs": 90, "total_size": "3mb", "total_size_in_bytes": 3248531, "total_time": "423ms", "total_time_in_millis": 423}, "search": {"fetch_current": 0, "fetch_time": "9.4s", "fetch_time_in_millis": 9449, "fetch_total": 5000, "groups": {"match": {"client_time": "0.63s", "client_time_in_millis": 627, "client_total": 100, "fetch_current": 0, "fetch_time": "302ms", "fetch_time_in_millis": 302, "fetch_total": 100, "query_current": 0, "query_time": "26ms", "query_time_in_millis": 26, "query_total": 100}, "match_abs": {"client_time": "0.34s", "client_time_in_millis": 342, "client_total": 100, "fetch_current": 0, "fetch_time": "195ms", "fetch_time_in_millis": 195, "fetch_total": 100, "query_current": 0, "query_time": "26ms", "query_time_in_millis": 26, "query_total": 100}, "match_no_rnd": {"client_time": "0.64s", "client_time_in_millis": 637, "client_total": 100, "fetch_current": 0, "fetch_time": "300ms", "fetch_time_in_millis": 300, "fetch_total": 100, "query_current": 0, "query_time": "35ms", "query_time_in_millis": 35, "query_total": 100}, "match_srt": {"client_time": "0.66s", "client_time_in_millis": 662, "client_total": 100, "fetch_current": 0, "fetch_time": "315ms", "fetch_time_in_millis": 315, "fetch_total": 100, "query_current": 0, "query_time": "29ms", "query_time_in_millis": 29, "query_total": 100}, "mlt": {"client_time": "0.12s", "client_time_in_millis": 122, "client_total": 100, "fetch_current": 0, "fetch_time": "1ms", "fetch_time_in_millis": 1, "fetch_total": 100, "query_current": 0, "query_time": "10ms", "query_time_in_millis": 10, "query_total": 100}}, "open_contexts": 0, "query_current": 0, "query_time": "2.1s", "query_time_in_millis": 2122, "query_total": 5000}, "store": {"size": "2.9mb", "size_in_bytes": 3091110, "throttle_time": "120.3ms", "throttle_time_in_millis": 120}}},"sort":[1385488714000]}]}}', curl='curl -XGET http://localhost:9200/stats/obs/_search?q=meta.benchmark_id:315fd1aa&sort=meta.observation_start:asc&size=10000')
        obs = list(esbench.analyze._observations(resp=esbench.analyze._get_observations()))
        self.assertEqual(2, len(obs))


    def test_get(self):
        for data in esbench.analyze.get_data(None):
            self.assertEqual(data.keys(), ['benchmark', 'observation'])


    def test_flatten_container(self):

        obs = [esbench.analyze.flatten_container(container=o) for o in esbench.analyze.get_data(conn=None)]
        self.assertEqual(len(obs), 2)
        self.assertEqual(len(obs[0]), len(obs[1]))

        obs = [[t for t in esbench.analyze.flatten_container(container=o) if t[0] == u'observation.stats.store.size'] for o in esbench.analyze.get_data(conn=None)]
        self.assertEqual(obs, [[(u'observation.stats.store.size', u'149.5kb')], [(u'observation.stats.store.size', u'293.9kb')]])


    def test_filter(self):
        for data in esbench.analyze.get_data(None):
            f = esbench.analyze.flatten_container(container=data)
            d = esbench.analyze.filter_tuples(f)
            self.assertEqual(sorted(f), d)
            self.assertEqual(esbench.analyze.filter_tuples(f, pattern='benchmark'), esbench.analyze.filter_tuples(f, pattern='benchmark.*'))


    def test_group_observations(self):
        data = list(esbench.analyze.get_data(None))
        # just check if the thing basically works
        benchmarks = esbench.analyze.group_observations(data=data, fields=esbench.analyze.FIELDS)
#         print(json.dumps(benchmarks, indent=4))
        self.assertEqual('observation.meta.observation_id',benchmarks[0][0][2][0])
        self.assertEqual('e8cd3f18',benchmarks[0][0][2][1])


class FlattenContainerTest(unittest.TestCase):

    def setUp(self):
        self.d = {'foo': [{'bar': 'baz'}, 2, 'monkey', ['foo', {'a': 1, 'b': 2, 'c': 3}], ]}
        self.a = ['foo', 'bar', {'monkey': True}, set(['a1', 'b1']), ('a2', 'b2')]

    def test_flatten(self):
        self.assertRaises(ValueError, esbench.analyze.flatten_container, container=iter(range(10)))
        f = esbench.analyze.flatten_container(container=self.d)
        self.assertEqual(sorted(f), [('foo.0.bar', 'baz'), ('foo.1', 2), ('foo.2', 'monkey'), ('foo.3.0', 'foo'), ('foo.3.1.a', 1), ('foo.3.1.b', 2), ('foo.3.1.c', 3)])
        f = esbench.analyze.flatten_container(container=self.a)
        self.assertEqual(sorted(f), [('0', 'foo'), ('1', 'bar'), ('2.monkey', True), ('3.0', 'a1'), ('3.1', 'b1'),  ('4.0', 'a2'), ('4.1', 'b2')])


if __name__ == "__main__":
    fmt = '%(levelname)s %(name)s.%(funcName)s:%(lineno)s %(message)s'
    logging.basicConfig(level=logging.DEBUG, format=fmt)
    logging.getLogger('esbench.analyze').setLevel(logging.ERROR)
    unittest.main()

